# -*- coding: utf-8 -*-
"""Loads, parses and returns values related to configuration

    Attributes:
        BRAG_DIR (str): The directory to use when reading and writing data.
            defaults to '~/.brag' (~/ is the user directory, and will be the
             correct value based on the platform. It can be overridden by the
             environmental variable BRAG_DIR.
        CONFIG_FILE_PATH (str): The file path for the configuration file.
            Defaults to $BRAG_DIR/config.ini, but can be overridden by setting
            the environmental variable BRAG_CONFIG_PATH.
"""
from __future__ import print_function, absolute_import
from six.moves import configparser
import os

BRAG_DIR = os.environ.get('BRAG_DIR', os.path.expanduser('~/.brag'))
CONFIG_FILE_PATH = os.environ.get('BRAG_CONFIG_PATH', os.path.join(BRAG_DIR, 'config.ini'))


def get_config(mechanism=None):
    """Gets the configuration from a file, if possible. If not, uses the
        default.

    Args:
        mechanism (str): The mechanism to use for persistance. One of 'files',
            'reldb' (relational database), or 'mongdb'.

    Returns:
        dict: A dictionary of configuration values relevant to the given
            persistence mechanism.
    """

    default_config = {
        'files': {
            'file_dir': BRAG_DIR,
            'form': 'log',
        },
    }

    conf_parser = configparser.ConfigParser()
    try:
        with open(CONFIG_FILE_PATH, 'r') as f:
            conf_parser.read_file(f)
    except IOError:
        print("No configuration found, using default configuration: ")

    if mechanism is None:
        if 'mechanism' in conf_parser:
            mechanisms = [
                mech for mech, onoff
                in conf_parser['mechanism'].items()
                if onoff.lower() == 'on'
            ]
            if len(mechanisms) > 1:
                raise RuntimeError('Multiple mechanisms are on. Please turn all'
                                   ' but one off.')
        else:
            mechanisms = ['files']
    else:
        mechanisms = [mechanism]

    config = {}
    for mechanism in mechanisms:
        if mechanism in conf_parser:
            overide_config = dict(conf_parser[mechanism])
        else:
            overide_config = {}

        # Sets the configuration as the default configuration, and then
        # overrides from the configuration file.
        config[mechanism] = dict(
            default_config.get(mechanism, {}),
            **overide_config
        )

    config[mechanism]['mechanism'] = mechanism
    return config[mechanism]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Entry point(s) for the command line interface"""
from bragly.cli import parse_args, parse_utility_args
import sys


def main(args=None):
    """Either takes arguments or gets them from sys.argv. Calls the bragly.cli
        parsing function parse_args, and then executes the returned function.
        See the usage at the command line by passing --help as an option.
    Args:
        args (list): A list of arguments that would be in sys.argv. If None,
            args will be populated from sys.argv.

    Returns:
        int:
            0 - If the program runs without exception.
            1 - If the program runs but there is an unexpected exception
            2 - If the program runs and no arguments are parsed form the command
                line.
    """
    if args is None:
        args = sys.argv[1:]
    parser, args = parse_args(args)
    if not args:
        parser.print_help()
        return 2

    func = args.pop('func', None)
    if func is None:
        print("Operation failed for unexpected reason")
        return 1
    results = func(**args)
    for result in results:
        print(result, flush=True)
    return 0

def util(args=None):
    """Entry point for brag utilities.

    Args:
        args (list): A list of arguments that would be in sys.argv. If None,
            args will be populated from sys.argv.

    Returns:

    """
    if args is None:
        args = sys.argv[1:]
    parser, args = parse_utility_args(args)
    if not args:
        parser.print_help()
        return 2

    func = args.pop('func', None)
    if func is None:
        print("Operation failed for unexpected reason")
        return 1
    results = func(**args)
    for result in results:
        print(result, flush=True)
    return 0


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""A module for writing, reading, and searching files for persistance of brags.
    Implements the three public methods necessary for the bragly.persist API:
        write : saves a message.
        read : reads all messages in the relevant timeframe.
        search: searches all messages according to a number of criteria

Attributes:
    MESSAGE_STR_TEMPLATE (str): A template string for the 'log' format.

"""

from __future__ import absolute_import, print_function
import os
import json
import re
from collections import namedtuple
import arrow

MESSAGE_STR_TEMPLATE = "[{timestamp}][{tags}] {message}\n"


def write(message, file_path=None, file_dir=None, form='json'):
    """Writes a message to a file in the given format.

    Args:
        message (dict): A message, it's timestamp, and any associated tags.
        form (str): A message format, one of log, json, json-pretty
        file_path (str): The file path for the source file. If None, the
            file_path will be generated from the file_dir and the form
        file_dir (str): The file directory for the source file. If None,
            the file_path will be used

    Returns:
        str: On success returns the word "success"
    """
    timestamp = message['timestamp'].isoformat()
    tags = message['tags']
    message = message['message']

    if form == 'json':
        message_str = "{}\n".format(json.dumps(
            dict(
                message=message,
                tags=tags,
                timestamp=timestamp
            ),
        ))
    elif form == 'json-pretty':
        message_str = "{}\n".format(json.dumps(
            dict(
                message=message,
                tags=tags,
                timestamp=timestamp
            ),
            indent=2
        ))
    elif form == 'log':
        tags = '|'.join(tags)
        message_str = MESSAGE_STR_TEMPLATE.format(
            timestamp=timestamp,
            tags=tags,
            message=message,
        )
    else:
        raise RuntimeError('Form {form} not supported or missing.'.format(form))

    file_path = _get_file_path(form, file_path, file_dir)

    with open(file_path, 'a') as f:
        f.write(message_str)

    return "success"


def _get_file_path(form, file_path=None, file_dir=None):
    """Given a format, and either a filepath or file directory, returns the
        proper filepath.
        The format of the file, if dynamically generated from the file_dir
            will be: /path/to/file/dir/brag-{format}.dat
    Args:
        form (str): A message format, one of log, json, json-pretty
        file_path (str): The file path for the source file. If None, the
            file_path will be generated from the file_dir and the form
        file_dir (str): The file directory for the source file. If None, the
            file_path will be used

    Returns:
        str: A file path for the source file
    """
    if file_path is None and file_dir is None:
        raise RuntimeError('No file_path or file_dir indicated.')

    if file_path is None:
        file_path = os.path.join(file_dir, 'brag-{form}.dat'.format(form=form))

    return file_path


def read(start, end, out_form, form, file_dir=None, file_path=None):
    """Reads out entries within the given time frame.

    Args:
        start (arrow.Arrow): The start date time to look for messages
        end (arrow.Arrow): The end date time to look for messages
        out_form (str): The output format. One of log, json, json-pretty
        form (str): The format that the source file is in
        file_dir (str): The file directory for the source file. If None, the
            file_path will be used.
        file_path (str): The file path for the source file. If None, the
            file_path will be generated from the file_dir and the form.
    Yields:
        str: A message line, in the requested format
    """
    file_path = _get_file_path(form, file_path, file_dir)

    if form == 'json-pretty':
        raise NotImplementedError('json-pretty format is not yet supported')
    with open(file_path, 'rb') as f:
        for line in f:
            line = line.decode('utf-8').strip()
            parsed_line = _parse_line(line, form=form)

            if ((start is None and parsed_line.timestamp <= end) or
                    (start is not None and
                        start <= parsed_line.timestamp <= end
                     )):
                if out_form != form:
                    yield _coerce_line(parsed_line, out_form)
                else:
                    yield line


ParsedLine = namedtuple('ParsedLine', ['timestamp', 'tags', 'message'])


def _parse_line(line, form):
    """Parse a message line according to the form that is passed in, returning
        a ParsedLine object.

    Args:
        line (str): A line from the data file
        form (str): Indicates the form that the line is in, one of log or json
            (json-pretty not supported)

    Returns:
        ParsedLine: The message in a convenient namedtuple
    """
    if form == 'log':
        line_regex = re.compile(r'\[(.*)\]\[(.*)] (.*)')
        timestamp, tags, message = line_regex.findall(line)[0]
        timestamp = arrow.get(timestamp)
        if not tags:
            tags = []
        else:
            tags = tags.split('|')
        message = message.strip()
        return ParsedLine(timestamp, tags, message)

    elif form == 'json':
        message_json = json.loads(line)
        return ParsedLine(
            arrow.get(message_json['timestamp']),
            message_json['tags'],
            message_json['message']
        )

    elif form == 'json-pretty':
        raise RuntimeError('No!')


def _coerce_line(parsed_line, out_form):
    """ Coerce's a parsed line (named tuple) into the requested output format.

    Args:
        parsed_line (ParsedLine): A named tuple of the message.
        out_form (str): The output format, one of log, json, json-pretty

    Returns:
        str OR ParsedLine: The line in the format requested
    """
    if out_form == 'parsed_line':
        return parsed_line
    timestamp = parsed_line.timestamp.isoformat()
    if out_form == 'log':
        tags = '|'.join(parsed_line.tags)
        return MESSAGE_STR_TEMPLATE.format(
            timestamp=timestamp,
            tags=tags,
            message=parsed_line.message
        ).strip()

    elif out_form == 'json':
        # Translate from a named tuple to a dict, and then dump as a json string
        return json.dumps({
            'timestamp': timestamp,
            'tags': parsed_line.tags,
            'message': parsed_line.message
        }).strip()

    elif out_form == 'json-pretty':
        # Translate from a named tuple to a dict, and then dump as a json string
        return json.dumps({
            'timestamp': timestamp,
            'tags': parsed_line.tags,
            'message': parsed_line.message
        }, indent=2).strip()


def search(start, end, out_form, tags, text,
           all_args, form, file_dir=None, file_path=None):
    """Given a file path or a file directory and form, searches the files
        according to the search parameters.

    Args:
        start (arrow.Arrow): The start date time to look for messages
        end (arrow.Arrow): The end date time to look for messages
        out_form (str): The output format. One of log, json, json-pretty
        tags (list): A list of tags to look for
        text (list): A list of text tokens (words) to look for
        all_args (bool): Indicates whether all tags and text must exist for the
            message to be surfaces
        form (str): The format that the source file is in
        file_dir (str): The file directory for the source file. If None, the
            file_path will be used.
        file_path (str): The file path for the source file. If None, the
            file_path will be generated from the file_dir and the form.

    Yields:
        str: A line of text representing one result of the search
    """
    base_results = read(start, end, 'parsed_line', form, file_dir, file_path)
    for result in base_results:
        if not all_args:
            if tags and set(tags).intersection(set(result.tags)):
                yield _coerce_line(result, out_form)
            elif text and set(text).intersection(
                    set(result.message.split(' '))):
                yield _coerce_line(result, out_form)
            elif not text and not tags:
                yield _coerce_line(result, out_form)
        else:
            tags_in_tags = False
            text_in_message = False
            if tags and set(tags).issubset(set(result.tags)):
                tags_in_tags = True
            elif not tags:
                tags_in_tags = True
            if text and set(text).issubset(set(result.message.split(' '))):
                text_in_message = True
            elif not text:
                text_in_message = True
            if tags_in_tags and text_in_message:
                yield _coerce_line(result, out_form)



# -*- coding: utf-8 -*-
"""A module to munge arguments comming from a front end and pass those requests
    to the persistence module.
"""
from __future__ import absolute_import, print_function
import arrow
import bragly.persist as persist
from bragly.config import BRAG_DIR, CONFIG_FILE_PATH
import os
import pkg_resources

PERSIST_CHOICES = list(persist.MECHANISMS.keys())

__all__ = [
    'write',
    'read',
    'search',
]

def write(message=None, tags=None, timestamp=None):
    """Given a message, tags, and timestamp, creates the proper structure for
        the persistance mechanism.

    Args:
        message: Can be one of None, list of strs or strs. If it is a list or
            None, it will be changed to a string.
        tags (list): A list of tags to associate with this message.
        timestamp (arrow.Arrow): An override date to associate with this message

    Returns:
        list: A list containing one element, representing the result status of
            the write operation.
    """
    if message is None:
        message = ''
    elif isinstance(message, (list,)):
        message = ' '.join(message)

    if tags is None:
        tags = []
    
    if timestamp is None:
        timestamp = arrow.now()

    message_struct = {'message': message, 'tags': tags, 'timestamp': timestamp}
    result = persist.write(message=message_struct)
    return [result]

def read(start=None, end=None, period=None, form='json'):
    """Reads previously saved messages.

    Args:
        start (arrow.Arrow): The start date time to read messages from
        end (arrow.Arrow): The end date time to read messages until
        period (str): A human readable description of a time period
        form (str): The output format, one of log, json, json-pretty

    Yields:
        str: A single line of the result set
    """
    end = _get_end_date(start, end, period)

    results = persist.read(start, end, form)
    for result in results:
        yield result

def search(start=None, end=None, period=None, form='json', tags=None, text=None, all_args=False):
    """Given search criteria, finds the matching messages. Yields the results
        back to the caller.

    Args:
        start (arrow.Arrow): The start date time to read messages from
        end (arrow.Arrow): The end date time to read messages until
        period (str): A human readable description of a time period
        form (str): The output format, one of log, json, json-pretty
        tags (list): A list of tags to search for
        text (list): A list of tokens (words) to search for
        all_args (bool): A flag indicating whether all tags and text must be
            present to return the message in the result.

    Yields:
        str: A single line of the result set
    """
    end = _get_end_date(start, end, period)

    if tags is None:
        tags = []
    if text is None:
        text = []

    results = persist.search(start, end, form, tags, text, all_args=all_args)
    for result in results:
        yield result

def _get_end_date(start=None, end=None, period=None):
    if end is None and period is None:
        end = arrow.now()
    elif end is None and start is not None:
        _, end = start.span(period)
    elif start is None:
        end = arrow.now()

    return end

def init(mechanism, clobber=True):
    directory = BRAG_DIR
    print('Checking if {} exists...'.format(directory), end='', flush=True)
    if not os.path.exists(directory):
        print('\nmaking directory...', end='', flush=True)
        os.makedirs(directory)
        print('success', flush=True)
    print('OK', flush=True)

    print('Getting example configuration for mechanism: {}...'.format(
        mechanism), end='', flush=True)

    config_example = pkg_resources.resource_filename(
        'bragly',
        'config_examples/config-{}.ini'.format(mechanism)
    )

    with open(config_example, 'r') as f:
        config_data = f.read()
    print('OK')

    config_file_path = CONFIG_FILE_PATH
    file_exists = os.path.exists(config_file_path)

    if (file_exists and clobber) or not file_exists:
        if file_exists:
            print('Clobbering file {}.'.format(config_file_path), flush=True)
        print('Writing to {}...'.format(config_file_path), end='', flush=True)
        with open(config_file_path, 'w') as f:
            f.write(config_data)
        print('OK')
    else:
        print(
            'Not clobbering file at {}. See example config file at {}.'.format(
                config_file_path, config_example
            )
        )

    return ['Initialization finished']



from setuptools import setup, find_packages
import os

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def brag_dir():
    return os.environ.get('BRAG_DIR', os.path.expanduser('~/.brag'))

with open('README.rst', 'r') as f:
    readme = f.read()


setup(
    name='bragly',
    version='0.1.0',
    description='A small tool to remind yourself of your day-to-day accomplishments',
    long_description = readme,
    url='http://github.com/huntcsg/bragly',
    author='Hunter Senft-Grupp',
    author_email='huntcgs@gmail.com',
    license='MIT',
    packages=find_packages(),
    entry_points={
       'console_scripts': [
            'brag = bragly.__main__:main',
            'brag-util = bragly.__main__:util'
        ],
    },
    install_requires=[
      'arrow',
      'six',
    ],
    package_data={'bragly': ['config_example/*.ini']},
    include_package_data=True,
    tests_require=[
        'nose',
        'coverage',
    ],
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# -*- coding: utf-8 -*-
"""A module that handles dispatching to the appropriate persistance module
    based on user configuration.

    Attributes:
        MECHANISMS (dict): A mapping from configuration string to persistance
            module.
"""

from __future__ import absolute_import, print_function
from bragly.persistance import files, reldb, mongodb
from bragly.config import get_config

MECHANISMS = {
    'files': files,
    'mongodb': mongodb,
    'reldb': reldb
}


def write(message, mechanism=None):
    """Gets the users configuration and dispatches the write call to the
        proper module and function.

    Args:
        message (dict): A dictionary representation of the message.
        mechanism (str): Allows for an override to explicitly get a mechanism
            from the configuration.

    Returns:
        str: A result message indicating success or failure
    """
    mech, config = _get_module_and_kwargs_from_config(mechanism)
    result = mech.write(message=message, **config)
    return result


def read(start, end, form='log', mechanism=None):
    """Given a start and end date, and an output format, gets the persistance
        module from the configuration and passes the request to that module.
        Yields the results back to the caller.

    Args:
        start (arrow.Arrow):
        end (arrow.Arrow):
        form (str): the output format. defaults to log.
        mechanism (str): Allows for an override to explicitly get a mechanism
            from the configuration.

    Yields:
        str: A single line of the result set
    """
    mech, config = _get_module_and_kwargs_from_config(mechanism)
    results = mech.read(start, end, form, **config)
    for result in results:
        yield result


def search(start, end, form, tags, text, all_args, mechanism=None):
    """Given search criteria, finds the matching messages. Yields the results
        back to the caller.

    Args:
        start (arrow.Arrow): The star tdate after which to find messages (inclusive)
        end (arrow.Arrow): The end date before which to find messages (inclusive)
        form (str): the output format. defaults to log
        tags (list): A list of tags to search for
        text (list): A list of text tokens (words) to search for
        all_args (bool): A flag indicating whether all tags and text must be
            present to return the message in the result.
        mechanism (str): Allows for an override to explicitly get a mechanism
            from the configuration

    Yields:
        str: A single line of the result set
    """
    mech, config = _get_module_and_kwargs_from_config(mechanism)
    results = mech.search(start, end, form, tags, text, all_args, **config)
    for result in results:
        yield result


def _get_module_and_kwargs_from_config(mechanism=None):
    """Optionally given a mechanism, get the user configuration. Using the
        configuration, find the correct module and return both to the caller

    Args:
        mechanism (str): The mechanism from MECHANISMS to get configuration for

    Returns:
        module, dict : The module and associated configuration for that module
    """
    config = get_config(mechanism)
    mechanism = config.pop('mechanism')
    mech = MECHANISMS[mechanism]
    return mech, config



# -*- coding: utf-8 -*-
"""Represents the needed functions and logic for a command line interface."""

import argparse
from bragly.brag import write, read, search, init, PERSIST_CHOICES
from bragly.config import BRAG_DIR, CONFIG_FILE_PATH
import arrow


def parse_args(args):
    """Parses arguments and dispatches the commands and argumnets to the
        appropriate sub parser using argparse.ArgumentParser.parse_args.
        Essentially, based on the commands given, returns a dictionary of
        options. One of the keys is "func" which is a function object
        from bragly.brag that maps to the given command.

    Args:
        args: The argments from sys.argv


    Returns:
        argpaser.ArgumentParser, dict: The parser object and a dictionary of
            options.
    """

    parser = argparse.ArgumentParser(prog='brag')
    subparsers = parser.add_subparsers(help='sub command help')

    # Writ command sub  parser
    write_parser = subparsers.add_parser('w', help='Write a new brag entry')
    write_parser.add_argument('message', metavar='message', nargs='+', type=str,
                              help='The brag message')
    write_parser.add_argument('-t', '--tags', nargs='*', type=str,
                              help='The tags associated with this brag message')
    write_parser.add_argument('-d', '--timestamp', type=arrow.get,
                              help='The time stamp to use for this entry, in ISO-8601 format')
    write_parser.set_defaults(func=write)

    # Read command sub parser
    read_parser = subparsers.add_parser('r',
                                        help='Read a group of brag entries')
    read_parser.add_argument('-s', '--start', type=arrow.get,
                             help="The start time for getting entries")
    # End date spec
    read_parser_enddate_group = read_parser.add_mutually_exclusive_group()
    read_parser_enddate_group.add_argument(
        '-p',
        '--period',
        type=str,
        help='The time period after the start datetime to get entires. One of hour, day, week, month, year'
    )
    read_parser_enddate_group.add_argument('-e', '--end', type=arrow.get,
                                           help='The end time for getting entries')
    # Other read options
    read_parser.add_argument(
        '-f',
        '--form',
        type=str,
        default='json',
        help='The format to display the results in. One of json, json-pretty, log. Default: %(default)s'
    )
    # Set the operation that will be called based on the command
    read_parser.set_defaults(func=read)

    # Search Command Sub parser
    search_parser = subparsers.add_parser('s',
                                          help='Search for a group of brag entries')
    search_parser.add_argument(
        '-s',
        '--start',
        type=arrow.get,
        help="The start time for getting entries",
    )
    # End date spec
    search_parser_enddate_group = search_parser.add_mutually_exclusive_group()
    search_parser_enddate_group.add_argument(
        '-p',
        '--period',
        type=str,
        help='The time period after the start datetime to get entires. One of hour, day, week, month, year'
    )
    search_parser_enddate_group.add_argument('-e', '--end', type=arrow.get,
                                             help='The end time for getting entries')
    # Other search options
    search_parser.add_argument('-t', '--tags', nargs='*', type=str,
                               help='Tags you want to search for')
    search_parser.add_argument('-x', '--text', nargs='*', type=str,
                               help='Keywords you want to search for')
    search_parser.add_argument(
        '-f',
        '--form',
        type=str,
        default='json',
        help='The format to display the results in. One of json, json-pretty, log. Default: %(default)s'
    )
    any_all_group = search_parser.add_mutually_exclusive_group()
    any_all_group.add_argument(
        '--any',
        action='store_true',
        help='Indicates that a result should be returned if it has at least one match in either tags or text.',
        default=True
    )
    any_all_group.add_argument(
        '--all',
        action='store_true',
        help='Indicates that a result should be returned if it matches all tags and text criteria.')

    # Set the operation that will be called based on the command
    search_parser.set_defaults(func=search)

    args = vars(parser.parse_args(args))
    # Do some additonal argument parsing if this is a search command
    if 'func' in args:
        if args['func'].__name__ == 'search':
            any_args = args.pop('any', True)
            all_args = args.pop('all', False)
            args['all_args'] = (not any_args) or all_args

    return parser, args


def parse_utility_args(args):
    parser = argparse.ArgumentParser(prog='brag-util')
    subparsers = parser.add_subparsers(help='sub command help')
    init_parser = subparsers.add_parser(
        'init',
        help='Initialize brag. If you want a different location for brag than'
             ' {} than be sure to set BRAG_DIR environmental variable. If you '
             'want a different location for the configuration file then be sure'
             ' to set BRAG_CONFIG_PATH to something other than {}'.format(
                    BRAG_DIR,
                    CONFIG_FILE_PATH,
                ),
    )
    init_parser.add_argument(
        '-m',
        '--mechanism',
        type=str,
        choices=PERSIST_CHOICES,
        default='files',
        help='Select the persistence mechanism. Default: %(default)s.'
    ),
    init_parser.add_argument(
        '-c',
        '--clobber',
        help='If set, overwrites existing configuration files.',
        action='store_true',
    ),

    init_parser.set_defaults(func=init)
    args = vars(parser.parse_args(args))

    return parser, args
